"""
Easy PTVSD Module.

Contains any decorators or convenience functions for PTVSD.
"""
import ptvsd


class wait_and_break:
    """
    Decorator to create ptvsd server, wait for attach, break into debugger, continue.

    This pattern of using a class to make a decorator cleans up the double nested
    functions needed to create a decorator that accepts arguments. Maybe a little hacky
    but I believe this definitely follows the zen of python more than a double nested
    decorator function.

    Essentially, when the decorator is evaluated it returns a new wait_and_break object
    that will be invoked with the decorated function as an argument and return the new
    wrapped function.
    """

    DEFAULT_SECRET = "my_secret"
    DEFAULT_ADDRESS = ("0.0.0.0", 3000)

    def __init__(self, secret=DEFAULT_SECRET, address=DEFAULT_ADDRESS):
        """Set default secret and address."""
        self.secret = secret
        self.address = address

    def __call__(self, function):
        """Run ptvsd code and continue with decorated function."""
        def wait_and_break_deco(*args, **kwargs):
            ptvsd.enable_attach(self.secret, address=self.address)
            ptvsd.wait_for_attach()
            ptvsd.break_into_debugger()
            function(*args, **kwargs)
        return wait_and_break_deco



"""Setup.py file for easy_ptvsd."""
from setuptools import setup

setup(
    name="easy_ptvsd",
    version="0.1.0",
    description="A convenience package for PTVSD.",
    long_description=(
        "EasyPtvsd is a convenience library that makes it a bit easy to remote"
        + " debug with ptvsd without having to remember the function calls and all"
        + " that. It currently provides a decorator that can be used to setup the ptvsd"
        + " server, wait for attachment, then immediately break into the debugger."
    ),
    url="https://github.com/colinfike/easy-ptvsd",
    author="Colin Fike",
    author_email="colin.fike@gmail.com",
    license="MIT",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'Programming Language :: Python :: 3',
    ],
    keywords="ptvsd easy python remote debugging",
    install_requires=[
        'ptvsd==3.0.0',
    ],
    python_requires='>=3',
    py_modules=["easy_ptvsd"],
    packages=[],
)
